package com.nemrodsoftware.maven;

/*
 * Copyright 2001-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import com.nemrodsoftware.INemrodClient;
import com.nemrodsoftware.NemrodClient;
import com.nemrodsoftware.exception.OperationException;
import com.nemrodsoftware.model.Locale;
import com.nemrodsoftware.model.Module;
import com.nemrodsoftware.model.Project;

/**
 * Goal which import resource file from a Nemrod instance
 *
 * @goal import
 * 
 * @phase process-sources
 */
public class ImportMojo
    extends AbstractMojo
{
    /**
     * Source folder.
     * @parameter expression="${import.srcfolder}" default-value="${basedir}/src/main/resources"
     */
    private File srcfolder;
    
    /**
     * Destination folder.
     * @parameter expression="${import.workfolder}" default-value="${project.build.outputDirectory}"
     */
    private File workfolder;
    
    /**
     * Login.
     * @parameter expression="${import.apikey}" 
     * @required
     */
    private String apikey;

    /**
     * Url.
     * @parameter expression="${import.url}"  default-value="http://live.localizeyourapps.com"
     * @required
     */
    private String url;  
    
    /**
     * Project name.
     * @parameter expression="${import.project}"
     * @required
     */
    private String projectName;        

    /**
     * to avoid exporting empty values
     * @parameter expression="${import.exportEmptyValues}" default-value="true"
     * @required
     */
    private Boolean exportEmptyValues;
    
    /**
     * to export only approved values
     * @parameter expression="${import.exportOnlyApprovedValues}" default-value="false"
     * @required
     */
    private Boolean exportOnlyApprovedValues;
    
    /**
     * export format
     *
     * @parameter expression="${import.format}" default-value="properties"
     * @required
     */
    private String format = "properties";        

    @Override
    public void execute()
        throws MojoExecutionException
    {
        try
        {
            INemrodClient client = new NemrodClient(url, apikey);
            
            getLog().info("Action : import ");
            getLog().info("url="+url);
            getLog().info("project="+projectName);
            getLog().info("format="+format);
            getLog().info("workfolder="+workfolder.getAbsolutePath());
            
            if (StringUtils.isEmpty(projectName))
                throw new IllegalArgumentException("Project name is not set. Please set nemrod.project property");
            
            Project project = client.viewProject(projectName);
            List<Locale> locales = retrieveSupportedLocales(client);
            List<Module> modules = retrieveModules(client);
            
            for (Module module : modules)
            {
                for (Locale supportedLocale : locales)
                {
                    getLog().info("");
                    getLog().info("Import bundle ["+module.getName()+ "] at path ["+module.getFullpath()+"] for locale : "+supportedLocale+ "...");
                    
                    execute(client,supportedLocale,module, project);
                }
            } 
        }
        catch (Exception e)
        {
            throw new MojoExecutionException("Unable to import resource bundles",e);
        }
    }

    private void updateDefaultBundle(Project project, Module module, File updatedFile) throws IOException {
        String srcFolder = srcfolder.getAbsolutePath();
        // FIXME the extension should be guessed from the format. What if my bundle is a resx file ?
        File target = new File (new File(srcFolder,module.getFullpath()),module.getName()+".properties");
        
        FileUtils.copyFile(updatedFile, target);
        getLog().info("");
        getLog().info("Update default bundle="+target.getAbsolutePath()+" - OK");
    }

    private List<Module> retrieveModules(INemrodClient client) {
        return client.listModule(projectName);
    }

    private List<Locale> retrieveSupportedLocales(INemrodClient client) {
        List<Locale> locales = client.listLocale(projectName);
        getLog().info("This project support the following locales : " + locales);
        return locales;
    }
    
    public void execute(INemrodClient client, Locale localeName, Module module, Project project) throws IOException, OperationException
    {
        String exportType = format;
        String workFolder = workfolder.getAbsolutePath();
        
        getLog().debug("");
        getLog().debug("Remote call with parameters : \n [" + projectName + "] " 
                                                     + "["+localeName+"] "
                                                     + "["+module.getName()+"] "
                                                     + "["+module.getFullpath()+"] "
                                                     + "["+exportType+"] " 
                                                     + "["+ exportEmptyValues+"] "
                                                     + "["+ exportOnlyApprovedValues+"] "
                                                     + "["+ workFolder+"]");
        getLog().debug("");
        File file = client.export(projectName, localeName.getIsoName(), module, exportType, exportEmptyValues, exportOnlyApprovedValues, workFolder);
        
        getLog().info("Download file="+file.getAbsolutePath()+" - OK");
        
        if (localeName.getIsoName().equalsIgnoreCase(project.getLocaleByDefault().getIsoName()))
        {
            updateDefaultBundle(project, module, file);
        }
    }    
}
